/****************************************************************************
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#ifndef LIFE_H
#define LIFE_H

#include <qframe.h>


class LifeWidget : public QFrame
{
    Q_OBJECT
public:
    LifeWidget( int s = 10, QWidget *parent = 0, const char *name = 0 );

    void	setPoint( int i, int j );

    int		maxCol() { return maxi; }
    int		maxRow() { return maxj; }

public slots:
    void	nextGeneration();
    void	clear();
	void	evaluate();
	void	gamain();
	void	initialize(void);
	void	keep_the_best(void);
	void	elitist(void);
	void	Select(void);
	void	crossover(void);
	void	Xover(int,int);
	void	swap(int *, int *);
	void	mutate(void);
	void	report(void);

protected:
    virtual void paintEvent( QPaintEvent * );
    virtual void mouseMoveEvent( QMouseEvent * );
    virtual void mousePressEvent( QMouseEvent * );
    virtual void resizeEvent( QResizeEvent * );
    void	 mouseHandle( const QPoint &pos );

private:
    enum { MAXSIZE = 50, MINSIZE = 10, BORDER = 5 };

    bool	cells[2][MAXSIZE + 2][MAXSIZE + 2];
    bool	init_cells[MAXSIZE + 2][MAXSIZE + 2];
	int		current;
    int		maxi, maxj;

    int pos2index( int x )
    {
	return ( x - BORDER ) / SCALE + 1;
    }
    int index2pos( int i )
    {
	return	( i - 1 ) * SCALE + BORDER;
    }

    int SCALE;
};

#define PI			3.14159265
#define	NRUNS		2      	/* Number of Runs */
#define POPSIZE 	25		    /* Population size */
#define MAXGENS 	20	    /* Max. number of generations */
#define PXOVER 		0.25	    /* Probability of crossover */
#define PMUTATION 	0.01	    /* Probability of mutation */
#define TRUE 		1
#define FALSE 		0
#define OPTIMAL		2.850273768
#define NVARS		9

#define MAX_FIT		22.5
#define MAX_DIFF	45
#define MAX_GENERATION	10


/* Declaration of procedures used by this genetic algorithm */

float Diff(bool *,bool *);

static	int	rule[9];
static	bool repaint_on;
static	int num_of_init;

#endif // LIFE_H
